'UpdateLicence.vbs
'Description : Update the licence of the EPOS P programming tool OpenPCS
'Author		 : Roland Odermatt, maxon motor ag
'Date		 : 07.04.2014
'==========================================================================

'================================================================================
'Get Date of Licence Expiration
'================================================================================
Function ExpirationDate(strSerialNumber,strLicenceCode)
Dim strMonth,strYear,strDate

	strMonth = Mid(strLicenceCode,1,2)
	strYear = Mid(strLicenceCode,3,2)
	
	If strMonth = "00" and strYear = "00" Then
		strDate = "Unrestricted"
	Else
		strDate = "14." & strMonth & "." & "20" & strYear
	End If

	ExpirationDate = strDate
End Function

Function ExpirationMsg(strDate)
Dim msg

	If strDate = "Unrestricted" Then
		msg = "Updated Licence is valid"
	Else
		msg = "Updated Licence is valid until " & expDate
	End If

	ExpirationMsg = msg
End Function

'================================================================================
'Check if Licence is valid 
'================================================================================
Function IsLicenceValid(strSerialNumber,strLicenceCode)
Dim expDate,today

	today = Date()
	expDate = ExpirationDate(strSerialNumber,strLicenceCode)

	IsLicenceValid = DateDiff("d",today,expDate) > 0
End Function

'================================================================================
'Check if Licence is from Maxon
'================================================================================
Function IsMaxonLicence(strSerialNumber,strLicenceCode)
Const MAXONOEMTAG = "592"
Dim oemTag

	oemTag = Mid(strSerialNumber,1,3)
	IsMaxonLicence = oemTag = MAXONOEMTAG
End Function

'================================================================================
'Check if Registry Key exists
'================================================================================
Function RegistryKeyExists(RegistryKey)

	'Create Shell Object
	set wshshell = createobject("WScript.shell")
		
	On Error Resume Next
	
	'Try reading the key
	wshshell.regread(RegistryKey)
	
	'Catch the error
	Select Case Err
		Case 0:
			RegistryKeyExists = true
		Case Else:
			RegistryKeyExists = false
	End Select
	
	On Error Goto 0

End Function

'================================================================================
'Get Registry Key
'================================================================================
Function RegistryKeyOpenPCS()
'Constants
Const keyOpenPCS_Win32 = "HKLM\SOFTWARE\infoteam Software GmbH\OpenPCS\"
Const keyOpenPCS_Win64 = "HKLM\SOFTWARE\Wow6432Node\infoteam Software GmbH\OpenPCS\"

	If RegistryKeyExists(keyOpenPCS_Win32) Then
		'Check for Win32 Registry Key
		RegistryKeyOpenPCS = keyOpenPCS_Win32
	Elseif RegistryKeyExists(keyOpenPCS_Win64) Then
		'Check for Win64 Registry Key
		RegistryKeyOpenPCS = keyOpenPCS_Win64
	Else
		RegistryKeyOpenPCS = ""
	End If
End Function

'================================================================================
'Check OpenPCS Version
'================================================================================
Sub CheckOpenPCSVersion(currentVersion)
	
	If (Left(currentVersion,2) = "5.") Then
		OpenPCS_Version = "2006"
	Elseif (Left(currentVersion,2) = "6.") Then
		OpenPCS_Version = "2008"
	Else
		OpenPCS_Version = ""
	End If

End Sub

'================================================================================
'Read Licences from Registry
'================================================================================
Sub GetLicences
'Variables
Dim keyOpenPCS
Dim currentVersion
Dim licenceNumber,serialNumber,licenceCode
Dim key,keySerialNumber,keyLicenceCode

	keyOpenPCS = RegistryKeyOpenPCS()
	If keyOpenPCS <> "" Then
			
		'Create Shell Object
		set wshshell = createobject("WScript.shell")
		
		'Prepare Base Key for Current Version
		currentVersion = wshshell.regread(keyOpenPCS & "CurrentVersion\")
		key = keyOpenPCS & currentVersion & "\Licences"
		
		'Check OpenPCS Version
		CheckOpenPCSVersion(currentVersion)

		'Switch Off Error Aborts
		On Error Resume Next

		'Read Licences
		For licenceNumber=1 To MaxNbOfLicences
			
			'Read SerialNumber
			SerialNumberArray(licenceNumber) = ""
			serialNumber = ""
			keySerialNumber = key & "\SerialNumber" & licenceNumber
			serialNumber = wshshell.regread(keySerialNumber)
			If serialNumber <> keySerialNumber Then
				SerialNumberArray(licenceNumber) = serialNumber
			End If

			'Read LicenceCode
			LicenceCodeArray(licenceNumber) = ""
			licenceCode = ""
			keyLicenceCode = key & "\LicenceCode" & licenceNumber
			licenceCode = wshshell.regread(keyLicenceCode)
			If licenceCode <> keyLicenceCode Then
				LicenceCodeArray(licenceNumber) = licenceCode
			End If

			'Update NbOfLicences
			If Err.Number = 0 Then
				NbOfLicences = NbOfLicences + 1
			End If
		Next
	End If

	'Switch On Error Aborts
	On Error GoTo 0
	
End Sub


'================================================================================
'Delete Invalid Maxon Licences and Add New Licence (OpenPCS 2006)
'================================================================================
Sub UpdateMaxonLicence_2006
'Variables
Dim licenceNumber,serialNumber,licenceCode
Dim showMsg,msg
Dim expDate
	
	'Init Variables
	licenceValid = false
	showMsg = true

	For licenceNumber=1 To MaxNbOfLicences
		
		'Get Values
		serialNumber = SerialNumberArray(licenceNumber)
		licenceCode = LicenceCodeArray(licenceNumber)
		
		'Check if Maxon Licence
		If IsMaxonLicence(serialNumber,licenceCode) Then
			
			'Reset Invalid Licence
			SerialNumberArray(licenceNumber) = ""
			LicenceCodeArray(licenceNumber) = ""
				
			'Set New Licence OpenPCS 2006
			If NbOfNewLicences < MaxNbOfNewLicences_2006 Then

				NbOfNewLicences = NbOfNewLicences + 1
				SerialNumberArray(licenceNumber) = NewSerialNumberArray_2006(NbOfNewLicences)
				LicenceCodeArray(licenceNumber) = NewLicenceCodeArray_2006(NbOfNewLicences)
			
				'ShowMessage
				If showMsg Then
					expDate = ExpirationDate(NewSerialNumberArray_2006(NbOfNewLicences),NewLicenceCodeArray_2006(NbOfNewLicences))
					msg = ExpirationMsg(expDate)
					WScript.Echo(msg)
					showMsg = false
				End If
			End If
		End If

		'Add Valid Licence to first empty Licence Entry
		If serialNumber = "" Then
			If NbOfNewLicences < MaxNbOfNewLicences_2006 Then
				
				NbOfNewLicences = NbOfNewLicences + 1
				SerialNumberArray(licenceNumber) = NewSerialNumberArray_2006(NbOfNewLicences)
				LicenceCodeArray(licenceNumber) = NewLicenceCodeArray_2006(NbOfNewLicences)				
				
				'ShowMessage
				If showMsg Then
					expDate = ExpirationDate(NewSerialNumberArray_2006(NbOfNewLicences),NewLicenceCodeArray_2006(NbOfNewLicences))
					msg = ExpirationMsg(expDate)
					WScript.Echo(msg)
					showMsg = false
				End If
				
				'Create New Licence Registry Entry
				If licenceNumber > NbOfLicences Then
					NbOfLicences = licenceNumber
				End If
			End If
		End If
	Next
	
End Sub

'================================================================================
'Delete Invalid Maxon Licences and Add New Licence (OpenPCS 2008)
'================================================================================
Sub UpdateMaxonLicence_2008
'Variables
Dim licenceNumber,serialNumber,licenceCode
Dim showMsg,msg
Dim expDate
	
	'Init Variables
	licenceValid = false
	showMsg = true

	For licenceNumber=1 To MaxNbOfLicences
		
		'Get Values
		serialNumber = SerialNumberArray(licenceNumber)
		licenceCode = LicenceCodeArray(licenceNumber)
		
		'Check if Maxon Licence
		If IsMaxonLicence(serialNumber,licenceCode) Then
			
			'Reset Invalid Licence
			SerialNumberArray(licenceNumber) = ""
			LicenceCodeArray(licenceNumber) = ""
				
			'Set New Licence OpenPCS 2008
			If NbOfNewLicences < MaxNbOfNewLicences_2008 Then

				NbOfNewLicences = NbOfNewLicences + 1
				SerialNumberArray(licenceNumber) = NewSerialNumberArray_2008(NbOfNewLicences)
				LicenceCodeArray(licenceNumber) = NewLicenceCodeArray_2008(NbOfNewLicences)
			
				'ShowMessage
				If showMsg Then
					expDate = ExpirationDate(NewSerialNumberArray_2008(NbOfNewLicences),NewLicenceCodeArray_2008(NbOfNewLicences))
					msg = ExpirationMsg(expDate)
					WScript.Echo(msg)
					showMsg = false
				End If
			End If
		End If

		'Add Valid Licence to first empty Licence Entry
		If serialNumber = "" Then
			If NbOfNewLicences < MaxNbOfNewLicences_2008 Then
				
				NbOfNewLicences = NbOfNewLicences + 1
				SerialNumberArray(licenceNumber) = NewSerialNumberArray_2008(NbOfNewLicences)
				LicenceCodeArray(licenceNumber) = NewLicenceCodeArray_2008(NbOfNewLicences)				
				
				'ShowMessage
				If showMsg Then
					expDate = ExpirationDate(NewSerialNumberArray_2008(NbOfNewLicences),NewLicenceCodeArray_2008(NbOfNewLicences))
					msg = ExpirationMsg(expDate)
					WScript.Echo(msg)
					showMsg = false
				End If
				
				'Create New Licence Registry Entry
				If licenceNumber > NbOfLicences Then
					NbOfLicences = licenceNumber
				End If
			End If
		End If
	Next
	
End Sub


'================================================================================
'Write Licences to Registry
'================================================================================
Sub SetLicences
'Variables
Dim keyOpenPCS
Dim currentVersion
Dim licenceNumber,serialNumber,licenceCode
Dim key,keySerialNumber,keyLicenceCode

	keyOpenPCS = RegistryKeyOpenPCS()
	If keyOpenPCS <> "" Then
	
		'Create Shell Object
		set wshshell = createobject("WScript.shell")
		
		'Prepare Base Key for Current Version
		currentVersion = wshshell.regread(keyOpenPCS & "CurrentVersion\")
		key = keyOpenPCS & currentVersion & "\Licences"

		'Switch Off Error Aborts
		On Error Resume Next

		'Read Licences
		For licenceNumber=1 To NbOfLicences
			
			'Write SerialNumber
			serialNumber = SerialNumberArray(licenceNumber)
			keySerialNumber = key & "\SerialNumber" & licenceNumber
			wshshell.regwrite keySerialNumber, serialNumber
			
			'Write LicenceCode
			licenceCode = LicenceCodeArray(licenceNumber)
			keyLicenceCode = key & "\LicenceCode" & licenceNumber
			wshshell.regwrite keyLicenceCode, licenceCode

		Next
	End If

	'Switch On Error Aborts
	On Error GoTo 0
	
End Sub

'Variables OpenPCS 2006
Dim MaxNbOfNewLicences_2006
Dim NewSerialNumberArray_2006(1)
Dim NewLicenceCodeArray_2006(1)

'Variables OpenPCS 2008
Dim MaxNbOfNewLicences_2008
Dim NewSerialNumberArray_2008(4)
Dim NewLicenceCodeArray_2008(4)

'Common Variables
Dim SerialNumberArray(20)
Dim LicenceCodeArray(20)
Dim MaxNbOfLicences,NbOfNewLicencs,NbOfLicences
Dim UpdateNecessary
Dim OpenPCS_Version

'Init New Licences OpenPCS 2006
MaxNbOfNewLicences_2006 = 1
NewSerialNumberArray_2006(1) = "592L012"
NewLicenceCodeArray_2006(1)  = "0000-ARM7_ARM-2LFE0K-0000-BS16JU3"

'Init New Licences OpenPCS 2008
MaxNbOfNewLicences_2008 = 4
NewSerialNumberArray_2008(1) = "592M025"
NewLicenceCodeArray_2008(1)  = "0000-ARM7_ARM-PUZ37G-0000-D3BD9C88-2KWSRD1"
NewSerialNumberArray_2008(2) = "592M026"
NewLicenceCodeArray_2008(2)  = "0000-ARM7_ARM-87H60G-0000-2C7CE26F-L1XED12"
NewSerialNumberArray_2008(3) = "592M027"
NewLicenceCodeArray_2008(3)  = "0000-ARM7_ARM-4UWEEU-0000-4C53B756-Z1F3WQ3"
NewSerialNumberArray_2008(4) = "592M028"
NewLicenceCodeArray_2008(4)  = "0000-ARM7_ARM-J9P373-0000-3B5487C0-8898170"

'Init Variables
MaxNbOfLicences = 20
NbOfLicences = 0
NbOfNewLicencs = 0
UpdateNecessary = True
OpenPCS_Version = ""

'Read Licences From Registry
GetLicences

'Update Licences
If OpenPCS_Version = "2006" Then
	UpdateMaxonLicence_2006
End If

If OpenPCS_Version = "2008" Then
	UpdateMaxonLicence_2008
End If

'Write Licences To Registry
SetLicences







